# -*- coding: utf-8 -*-
from core.libs import *
import unicodedata

HOST = 'https://www.hdtorrents.info'

LNG = Languages({
    Languages.es: ['Español'],
    Languages.la: ['Latino'],
    Languages.vos: ['VO']
})

QLT = Qualities({
    Qualities.rip: ['HDRip', 'hdrip', 'Hdrip'],
    Qualities.uhd: ['4K', '4k'],
    Qualities.hd: ['720p', 'hd720p'],
    Qualities.hd_full: ['1080p', 'MicroHD', 'hd1080p', 'microhd', 'Microhd', 'BluRay Rip', 'HD'],
    Qualities.m3d: ['3D', 'hd3d', '3d']
})


def mainlist(item):
    logger.trace()
    itemlist = list()

    itemlist.append(item.clone(
        action="movies",
        label="Novedades",
        type="item",
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="movies",
        label="Top ten torrents",
        url=HOST + '/torrents',
        type="item",
        content_type='movies'
    ))

    '''itemlist.append(item.clone(
        action="portada_sin_hilos",
        label="En portada (sin hilos)",
        url=HOST,
        type="item",
        content_type='movies'
    ))'''

    itemlist.append(item.clone(
        action="portada_con_hilos",
        label="En portada (con hilos)",
        url=HOST,
        type="item",
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="tags_qlt",
        label="Filtrar por calidad",
        url=HOST,
        type="item",
        category='movie'
    ))

    itemlist.append(item.clone(
        action="search",
        label="Buscar",
        query=True,
        type='search',
        content_type='movies'
    ))

    return itemlist


def tags_qlt(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url).data
    data = scrapertools.find_single_match(data, 'tagsdescargas">(.*?)header_info">')

    patron = '<a href="(?P<url>[^"]+)" rel="tag" class="">(?P<tag>[^<]+)</a></li>'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action="movies",
            label=result.group('tag'),
            url=result.group('url'),
            type='item',
            content_type='movies'
        ))

    return itemlist


def movies(item):
    logger.trace()
    itemlist = list()
    auxdict = dict()

    if not item.url:
        item.url = HOST + '/torrents'

    data = httptools.downloadpage(item.url).data
    data = re.sub(r"\n|\r|\t|\s{2}|&nbsp;", "", data)

    patron = 'class="%s\s?"><a(?: title="([^"]+)")? href="([^"]+)"(?: title="([^"]+)")?.*?src="([^"]+)"'

    if item.label == "Top ten torrents":
        agrupar = False
        patron = patron % "mostread"

    elif item.query:
        agrupar = True
        patron = patron % "moditemfdb"

    else:
        agrupar = True
        patron = patron % "blogitem"

    #logger.debug(patron)

    i = 0
    for title1, url, title2, poster in scrapertools.find_multiple_matches(data, patron):
        # Limpiamos el titulo de adjetivos (no he visto ningun ejemplo en novedades, pero si tu lo dices...)
        title = title1 or title2
        # Pasamos el patrón de sustitución a la función idioma_calidad(title)
        # title = re.sub(r" -| 3D|\*.*?\*| DTS", "", title).strip()
        #logger.debug(title)

        idioma, calidad, title = idioma_calidad(title)

        #logger.debug(title)
        #logger.debug(calidad)
        #logger.debug(idioma)

        new_item = item.clone(
                    action='findvideos',
                    title=title,
                    url=[HOST + url],
                    poster=poster.replace('t.jpg', '.jpg'),
                    type='movie',
                    quality=[QLT.get(calidad)],
                    lang= [LNG.get(idioma)],
                    content_type='servers'
                )

        if not agrupar:
            # No agrupar los titulos dentro de la misma pagina
            label_extra = {"sublabel": " -MicroHD-",
                           "color": "yellow",
                           "value": "True"} if calidad.lower() == 'microhd' else None

            auxdict[i] = (i, new_item.clone(label_extra=label_extra))
            i += 1

        else:
            # Agrupar los titulos dentro de la misma pagina mostrando las calidades e idiomas encontrados

            key = normalizar(title)
            # Al normalizar el titulo en minuscula agrupamos elementos q podrian ser iguales pero estar etiquetados
            # de forma distinta. Por ejemplo: "El Reino" = "El reino" o "12 dias" = "12 días"

            if not key in auxdict:
                auxdict[key] = (i, new_item.clone())
                i += 1

            else:
                orden, it = auxdict[key]
                q_aux = it.quality[:]
                l_aux = it.lang[:]
                u_aux = it.url[:]
                u_aux.append(HOST + url)

                if QLT.get(calidad) not in it.quality:
                    q_aux.append(QLT.get(calidad))

                if LNG.get(idioma) not in it.lang:
                    l_aux.append(LNG.get(idioma))

                auxdict[key] = (orden, it.clone(
                    url=u_aux,
                    lang=l_aux,
                    quality=sorted(q_aux, key=lambda q: q.level, reverse=True)
                ))


    for orden, it in sorted(auxdict.values(), key=lambda x: x[0]):
        # label_extra se elimina al clonar, por eso lo volvemos a introducir si es necesario
        itemlist.append(it.clone(label_extra=it.label_extra if it.label_extra else None))


    if item.label != "Top ten torrents":
        next_page = scrapertools.find_single_match(data, '<a href="(?P<next_page>[^"]+)" title="Siguiente">Siguiente</a>')
        if next_page and itemlist:
            itemlist.append(item.clone(
                url=HOST + next_page,
                label=item.label,
                type='next'
            ))

    return itemlist


def normalizar(s):
    """
    Devuelve la cadena s en minusculas y sin marcas diacríticas como tildes, dieresis, etc...
    :param s: str o unicode a normalizar
    :return: cadena sin marcas diacriticas
    """
    #import unicodedata
    s = s.lower()
    if isinstance(s, str):
        s = s.decode('utf-8')

    return ''.join((c for c in unicodedata.normalize('NFD',unicode(s)) if unicodedata.category(c) != 'Mn'))


def search(item):
    logger.trace()

    if not item.url:
        item.url = HOST + '/buscar?searchword=%s&ordering=&searchphrase=all&limit=40' % item.query.replace(" ", "+")

    return movies(item)


def idioma_calidad(title):
    # Obtenemos el idioma y la calidad del titulo
    # title: El Padrino (Latino) [HD]
    # return: Latino, HD, El Padrino

    # Usamos el patrón de sustitución para simplificar
    title = re.sub(r" -| 3D| \*.*?\*| DTS", "", title).strip()

    calidad = scrapertools.find_single_match(title, '\[([^\]]+)\]$')
    if calidad:
        title = title.replace(' [%s]' % calidad, '')
        # Podriamos convertirla en mimuscula para no tener q incluir variaciones
        # del tipo: 'HDRip', 'hdrip' o 'Hdrip' en la declaracion de calidades.
    else:
        calidad = "Desconocida"

    idioma = scrapertools.find_single_match(title, '\(([^\)]+)\)$')
    if idioma:
        title = title.replace(' (%s)' % idioma, '')
    else:
        idioma = 'Español'

    return idioma, calidad, title


"""def portada_sin_hilos(item):
    logger.trace()
    itemlist = list()
    auxdict = dict()
    n = time.time()

    data = httptools.downloadpage(item.url).data
    patron = '<a href="[^"]+" title="([^"]+)".*?src="([^"]+)"'

    # Obtenemos un listado de los titulos de la portada sin repeticiones
    for title, poster in scrapertools.find_multiple_matches(data, patron):
        idioma, calidad, title = idioma_calidad(title)
        key = normalizar(title)
        if not key in auxdict:
            auxdict[key]=item.clone(
                action='findvideos',
                title=title,
                poster=poster,
                type='movie',
                content_type='servers')

    # Por cada titulo encontrado lanzamos secuencialmente una busqueda
    for item in auxdict.values():
        sub_busqueda(item, itemlist)

    logger.debug("Sin hilos %s segundos" % (time.time() - n))
    return itemlist"""


def portada_con_hilos(item):
    logger.trace()
    itemlist = list()
    auxdict = dict()
    n = time.time()

    data = httptools.downloadpage(item.url).data
    patron = '<a href="([^"]+)" title="([^"]+)".*?src="([^"]+)"'

    # Obtenemos un listado de los titulos de la portada sin repeticiones
    for url, title, poster in scrapertools.find_multiple_matches(data, patron):
        idioma, calidad, title = idioma_calidad(title)
        key = normalizar(title)
        if not key in auxdict:
            auxdict[key]=item.clone(
                action='findvideos',
                title=title,
                poster=poster,
                type='movie',
                content_type='servers')

    # Por cada titulo encontrado lanzamos una busqueda (un hilo)
    threads = list()
    for item in auxdict.values():
        t = Thread(target=sub_busqueda, args=[item, itemlist])

        t.setDaemon(True)
        t.start()
        threads.append(t)

    # Esperamos que todas las busquedas (hilos) hayan terminado ...
    while [t for t in threads if t.isAlive()]:
        time.sleep(0.5)

    # ... y devolvemos la lista q se ha ido llenando en los diferentes hilos
    # En este caso el orden no es importante, pero en otros casos habra que tenerlo en cuenta
    # añadiendo un indice por ejemplo.
    logger.debug("Con hilos %s segundos" % (time.time() - n))
    return itemlist


def sub_busqueda(item, itemlist):
    list_urls = list()
    idiomas = set()
    calidades = set()

    # Si el title tiene menos de 3 caracteres no se hace una búsqueda de title ya que no devolvería resultados.
    if len(item.title) < 3:
        url = item.url
        patron = '<a href="(?P<url>[^"]+)" title="(?P<title>[^"]+)".*?src="[^"]+"'
    else:
        # Se limita la longitud de la búsqueda porque si tiene más de 50 caracteres la web no devuelve resultados
        url = HOST + '/buscar?searchword=%s&ordering=&searchphrase=all&limit=20' % item.title[:50]
        patron = 'class="moditemfdb"><a title="(?P<title>[^"]+)" href="(?P<url>[^"]+)"'

    data = httptools.downloadpage(url).data
    data = re.sub(r"\n|\r|\t|\s{2}|&nbsp;", "", data)
    # patron = 'class="moditemfdb"><a title="([^"]+)" href="([^"]+)"'

    # Pasamos a usar el método result para obtener title y url independientemente del patrón usado
    for result in re.compile(patron, re.DOTALL).finditer(data):
        #logger.debug(title)
        idioma, calidad, title = idioma_calidad(result.group('title'))
        #logger.debug('    %s, %s' %(idioma, calidad))

        if normalizar(title) == normalizar(item.title):
            # Añadimos la url a la lista
            list_urls.append(HOST + result.group('url'))
            # Creamos listados sin repeticion (conjunto set) con los diferentes idiomas y calidades encontradas.
            idiomas.add(LNG.get(idioma))
            calidades.add(QLT.get(calidad))
        ''' logger.debug('%s == %s' % (item.title, title))
        else:
            logger.debug('%s <> %s' % (item.title, title))'''


    if list_urls:
        itemlist.append(item.clone(
            url= list_urls,
            quality=sorted(list(calidades), key=lambda c: c.level, reverse=True),
            lang=list(idiomas)
        ))

    return itemlist


def findvideos(item):
    logger.trace()
    itemlist = list()

    for link in item.url:
        data = httptools.downloadpage(link).data
        data = re.sub(r"\n|\r|\t|\s{2}|&nbsp;", "", data)

        title, url = scrapertools.find_single_match(data, '<title>([^-]+)- HD Torrents</title>.*?'
                                                          'download"><p><a href="([^"]+)" rel=')
        title = re.sub(r" -| 3D|\*.*?\*| DTS", "", title).strip()
        # logger.debug(title)
        # logger.debug(url)

        idioma, calidad, title = idioma_calidad(title)

        # Tenemos 2 opciones: Usar la función url_torrent(url) o obtener el torrent directamente.
        torrent = url_torrent(url)
        # torrent = scrapertools.find_single_match(httptools.downloadpage(HOST + url).data, 'click <a href="([^"]+)"><strong>')
        # logger.debug(torrent)

        # Si el enlace contiene /c02/ la web no ha generado el torrent correctamente y no lo añadimos.
        if not '/c02/' in torrent:
            itemlist.append(item.clone(
                action='play',
                url=torrent,
                quality=QLT.get(calidad),
                lang=LNG.get(idioma),
                label_extra={"sublabel": " -MicroHD-",
                             "color": "yellow",
                             "value": "True"} if calidad.lower() == 'microhd' else None,
                server='torrent',
                type='server'
            ))

    # Si no ha encontrado torrents válidos, se informa al usuario.
    if len(itemlist) == 0:
        itemlist.append(item.clone(
            title=item.title,
            label_extra={"sublabel": " -El enlace no es válido-",
                         "color": "yellow",
                         "value": "True"},
            type='item'
        ))

    """
    if len(itemlist) == 1:
        return servertools.get_servers_from_id(itemlist)
    else:
        return servertools.get_servers_from_id(sorted(itemlist, key=lambda i: i.quality.level, reverse=True))
    """

    # Muestra los enlaces sin criterio de orden ya que existe la posibilidad de
    # ordenarlos en la configuración general de servidores.
    return servertools.get_servers_from_id(itemlist)


def url_torrent(url):
    logger.trace()

    data = httptools.downloadpage(HOST + url).data
    data = re.sub(r"\n|\r|\t|\s{2}|&nbsp;", "", data)
    torrent = scrapertools.find_single_match(data, 'click <a href="([^"]+)"><strong>')

    return torrent


